#ifndef IO_HEADER
#define IO_HEADER

#define SERVO_IN_2_DDR DDRB
#define SERVO_IN_2_PORT PORTB
#define SERVO_IN_2_PIN PINB
#define SERVO_IN_2_BIT 2
#define DRIVER_FAULT_PIN PINC
#define DRIVER_FAULT_BIT 2
#define DRIVER_ENABLE_PORT PORTC
#define DRIVER_ENABLE_BIT 1
#define LED_PORT PORTD
#define LED_BIT 2
#define SERVO_IN_1_DDR DDRE
#define SERVO_IN_1_PORT PORTE
#define SERVO_IN_1_PIN PINE
#define SERVO_IN_1_BIT 1

#define S1_PORT_IN PINE
#define S1_BIT 1
#define S2_PORT_IN PINE
#define S2_BIT 2
#define S3_PORT_IN PINC
#define S3_BIT 3

enum port_mode
{
	PM_OUT_0,
	PM_OUT_1,
	PM_IN,
	PM_IN_PULL
};

inline void pin_init(uint8_t *ddr, uint8_t *port,
		uint8_t bit, enum port_mode mode)
{
	switch(mode){
	case PM_OUT_0:
		*ddr |= (1<<bit);
		*port &= ~(1<<bit);
		break;
	case PM_OUT_1:
		*ddr |= (1<<bit);
		*port |= (1<<bit);
		break;
	case PM_IN:
		*ddr &= ~(1<<bit);
		*port &= ~(1<<bit);
		break;
	case PM_IN_PULL:
		*ddr &= ~(1<<bit);
		*port |= (1<<bit);
		break;
	default:
		break;
	}
}

inline void init_io(void)
{
	uint8_t ddrb=0, portb=0;
	uint8_t ddrc=0, portc=0;
	uint8_t ddrd=0, portd=0;
	uint8_t ddre=0, porte=0;

	// PORTB
	// 7: PSCOUT01: /L3
	pin_init(&ddrb, &portb, 7, PM_OUT_0);
	// 6: PSCOUT11: /L2
	pin_init(&ddrb, &portb, 6, PM_OUT_0);
	// 5: (INTX): SERVO_IN_1
	pin_init(&ddrb, &portb, 5, PM_IN);
	// 4: AMP0+: I_SENSE
	pin_init(&ddrb, &portb, 4, PM_IN);
	// 3: AMP0-: I_SENSE_GND
	pin_init(&ddrb, &portb, 3, PM_IN);
	// 2: (INT1): SERVO_IN_2
	pin_init(&ddrb, &portb, 2, PM_IN_PULL);
	// 1: PSCOUT21: /L1
	pin_init(&ddrb, &portb, 1, PM_OUT_0);
	// 0: PSCOUT20: /H1
	pin_init(&ddrb, &portb, 0, PM_OUT_0);

	// PORTC
	// 7: Digital-to-Analog DAC (D2A)
	pin_init(&ddrc, &portc, 7, PM_IN);
	// 6: ACMP1: VS2
	pin_init(&ddrc, &portc, 6, PM_IN);
	// 5: ADC9: TEMPERATURE
	pin_init(&ddrc, &portc, 4, PM_IN);
	// 4: ADC8: VOLTAGE
	pin_init(&ddrc, &portc, 5, PM_IN);
	// 3: S3
	pin_init(&ddrc, &portc, 3, PM_IN_PULL);
	// 2: /FAULT (fet driver)
	pin_init(&ddrc, &portc, 2, PM_IN_PULL);
	// 1: EN (fet driver)
	pin_init(&ddrc, &portc, 1, PM_OUT_0);
	// 0: PSCOUT10: /H2
	pin_init(&ddrc, &portc, 0, PM_OUT_0);

	// PORTD
	// 7: ACMP0: VS1
	pin_init(&ddrd, &portd, 7, PM_IN);
	// 6: ACMPM: GND
	pin_init(&ddrd, &portd, 6, PM_IN);
	// 5: ACMP2: VS3
	pin_init(&ddrd, &portd, 5, PM_IN);
	// 4: SCK / RXD
	pin_init(&ddrd, &portd, 4, PM_IN_PULL);
	// 4: SCK / RXD / SENSOR:ST
	//pin_init(&ddrd, &portd, 4, PM_IN);
	// 3: MOSI / TXD
	pin_init(&ddrd, &portd, 3, PM_OUT_0);
	//pin_init(&ddrd, &portd, 3, PM_IN_PULL);
	// 2: LED (MISO)
	pin_init(&ddrd, &portd, 2, PM_OUT_0);
	// 1: BUTTON
#define BUTTON_PIN PIND
#define BUTTON_BIT 1
	pin_init(&ddrd, &portd, 1, PM_IN_PULL);
	// 0: PSCOUT00: /H3
	pin_init(&ddrd, &portd, 0, PM_OUT_0);
	
	// PORTE
	// 2: S2
	//pin_init(&ddre, &porte, 2, PM_IN_PULL);
	pin_init(&ddre, &porte, 2, PM_IN);
	// 1: S1
	pin_init(&ddre, &porte, 1, PM_IN_PULL);
	// 0: /RESET
	pin_init(&ddre, &porte, 0, PM_IN_PULL);
	
	// Set values
	DDRB = ddrb;
	PORTB = portb;
	DDRC = ddrc;
	PORTC = portc;
	DDRD = ddrd;
	PORTD = portd;
	DDRE = ddre;
	PORTE = porte;
}

inline void led_on(void)
{
	LED_PORT |= (1<<LED_BIT);
}
inline void led_off(void)
{
	LED_PORT &= ~(1<<LED_BIT);
}
inline void led_toggle(void)
{
	LED_PORT ^= (1<<LED_BIT);
}
inline uint8_t button_pressed(void)
{
	return (BUTTON_PIN & (1<<BUTTON_BIT))?0:1;
}
inline void driver_enable(void)
{
	DRIVER_ENABLE_PORT |= (1<<DRIVER_ENABLE_BIT);
}
inline void driver_disable(void)
{
	DRIVER_ENABLE_PORT &= ~(1<<DRIVER_ENABLE_BIT);
}
inline uint8_t driver_fault(void)
{
	return (DRIVER_FAULT_PIN & (1<<DRIVER_FAULT_BIT))?0:1;
}

#endif

